 /*

********************************************************************************
**	ATI Mach 64 chipset driver
********************************************************************************
**
    Copyright (C)	1998	Isaac Connor
				Mathew Wilkins

********************************************************************************
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
********************************************************************************
*/

/*-----------------------------------------------------------------------------
 *	Kgi Managment Section - The Compulsory Bits.
 */

#ifndef _MACH64_C__
#define _MACH64_C__

#define  MAINTAINER	Isaac_Connor
#define  DRIVER_NAME "ATI Mach64 chipset driver"
#define  DRIVER_REV  "$Revision: 1.10 $"
#define DEBUG_LEVEL 2

#include <kgi/module.h>

#include <ATI/mach64.h>
#include "ATI/mach64_detect.h"
#include "ATI/mach64_init.h"
#include "ATI/mach64_modes.h"
#include "ATI/mach64_io.h"

struct kgi_chipset Mach64_Chip_Info = {
		"ATI Technologies Inc.",	/* Manufacturer */
                "Mach64",			/* Model */
                GM_8BIT | GM_15BIT | GM_16BIT	/* supported modes */ 
			| GM_24BIT | GM_32BIT,
                0,				/* flags */
                { 2040, 1280 },			/* max resolution */
		NULL,				/* hook for private data */
                16 MB,				/* addressable memory, check?*/
                { 10000000, 220000000 }		/* DCLK limits */
        };

kgi_sint kgim_chipset_init( struct kgi_card *Card, struct kgi_display *dpy ) {
  
    info.cards = NULL;
    if ( EOK == mach64_detect( dpy ) )
	return 	mach64_init( dpy );
    
    return -E(CHIPSET, UNKNOWN);
}



void	kgim_chipset_done( struct kgi_display *dpy ) {
/* Unused at this time */
/*    int count = dpy->id; */
    GET_PRIVATE;

/* Where does card come from? */
    if ( ! card) {
	printk( KERN_ERR "Request to shutdown non-existant display!: %d\n",
			    			dpy->id );
	return;
    }
    mach64_restore_card( dpy );
    printk( KERN_ERR "kgim_chipset_done: chipset driver removed.\n" );
    /* Put in linked card list removal etc. */
}

kgi_sint kgim_chipset_check_mode ( struct kgi_display *dpy,
				   struct kgi_mode *mode, 
     				   enum kgi_tm_cmd *cmd ) {
	switch( *cmd ) {
		case CMD_PROPOSE:

			return	mach64_propose_mode( dpy, mode );

		case CMD_LOWER:
		case CMD_RAISE:
		case CMD_CHECK:
			return	mach64_check_mode( dpy, mode );

		default:
			printk( KERN_ERR "Internal Error! Bad cmd \
						number: %d.\n", *cmd );

	}/* switch */

	return -E( CHIPSET, FAILED );
}




void	kgim_chipset_set_mode( struct kgi_display *dpy, struct kgi_mode *mode ) 
{
GET_PRIVATE;
int bpp_const, bpp;
kgiu32 cnt,cnt2, temp;

    switch( mode->request.graphtype ) {
        case	KGIGT_TEXT16:
		break;
	case    KGIGT_4BIT:  bpp_const = 1;bpp = 4; break;
	case	KGIGT_8BIT:  bpp_const = 2;bpp = 8; break;
	case	KGIGT_15BIT: bpp_const = 3;bpp = 16;break;
	case	KGIGT_16BIT: bpp_const = 4;bpp = 16;break;
	case	KGIGT_24BIT: bpp_const = 5;bpp = 24;break;
	case	KGIGT_32BIT: bpp_const = 6;bpp = 32;break;

	break;

	default:
	break;

    }/* switch */
		
	/* set the mode as requested */
	/*
	 * Crtc_Gen_Cntl settings:  
	 * CRTC_PIX_WIDTH   = 2    -  (8bpp)
	 * CRTC_LOCK_REGS   = 0    -  this has no apparent effect.
	 * CRTC_ENABLE	    = 1    -  otherwise we get no display.
	 * CRTC_EXT_DISP_EN = 1    -  turn on extended (non-VGA) mode
	 * CRTC_DISP_REQ_EN = 0    -  IMPORTANT.
	 * CRTC_FIFO_LWM    = 0xE  -  just a guess.
	 */
	mach64_out32(Crtc_Gen_Cntl, (3 << 24) | (0xE << 16) | (2 << 8), dpy); 
	mach64_out32(Crtc_H_Total, (mode->x.total >> 3) | 
			(((mode->x.width >> 3)-1) << 16), dpy);
	mach64_out32(Crtc_H_Sync, (mode->x.syncstart/8) |
		((mode->x.syncstart % 8)<< 8) | 	
		(((mode->x.syncend -mode->x.syncstart)/8) << 16 ) |
		(mode->x.polarity << 21), dpy);
	mach64_out32(Crtc_V_Total, (mode->y.total)  | 
			((mode->y.width-1)  << 16), dpy);
	mach64_out32(Crtc_V_Sync, (mode->y.syncstart) | 
		(mode->y.syncend - mode->y.syncstart) << 16 |
		(mode->y.polarity << 21), dpy);

	/* set display start and pitch. */
	mach64_out32(Crtc_Display_Start, (mode->request.virt.x/8) <<22 , dpy);
/*	mach64_out32(Crtc_Int_Cntl, 0 , dpy); */

	mode->fsize = ( mode->request.virt.x *
	mode->request.virt.y * (bpp >> 3)) ;
	mode->fb_size = (mode->fsize + 0xFFFF) & ~0xFFFF;
	temp = mach64_in32(Crtc_Display_Start, dpy);
	DEBUG1("Display start: 0x%x pixels", (temp & 0xFFFFF) << 3); 
	DEBUG1("Display pitch: 0x%x pixels", (temp & 0xFFE00000) >> 19);
	return;
}




kgi_sint	kgim_chipset_command
( struct kgi_graphic *graph, kgi_uint cmd, kgi_uint arg )
{
	printk( KERN_ERR "kgim_chipset_command: command refused.\n" );
	return	-E( CHIPSET, UNKNOWN );
}

void kgim_chipset_free_mode( struct kgi_mode *mode) {

}
#endif
